@@ -1,5 +1,6 @@
 import numpy as np
 import pandas as pd
+import warnings
 from scipy.spatial import distance_matrix
 from scipy.spatial.distance import pdist, squareform
 import tsfresh
@@ -8,7 +9,7 @@ import matplotlib.pyplot as plt
 from pyod.models.knn import KNN
 from fastdtw import fastdtw
 from scipy.spatial.distance import euclidean
-from tslearn.clustering import TimeSeriesKMeans
+from geoalchemy2 import functions, elements
 
 
 def grouping_data(processed_data, pick_vars=None):
@@ -66,7 +67,7 @@ def regrouping_data(data_animal_id_groups):
     result = pd.concat(data_animal_id_groups[aid]
                        for aid in data_animal_id_groups.keys())
 
-    result.sort_values(['time', 'animal_id'], ascending=True, inplace=True)
+    result.sort_values(['animal_id', 'time'], ascending=True, inplace=True)
     # Reset index-
     result.reset_index(drop=True, inplace=True)
     return result
@@ -131,20 +132,27 @@ def compute_distance_and_direction(data_animal_id_groups):
     """
     # Compute 'direction' for 'animal_id' groups-
     for aid in data_animal_id_groups.keys():
-        data_animal_id_groups[aid]['direction'] = np.rad2deg(
-            np.arctan2((data_animal_id_groups[aid]['y'] -
-                        data_animal_id_groups[aid]['y'].shift(periods=1)),
-                       (data_animal_id_groups[aid]['x'] -
-                        data_animal_id_groups[aid]['x'].shift(periods=1))))
+        try:
+            data_animal_id_groups[aid]['direction'] = np.rad2deg(
+                np.arctan2((data_animal_id_groups[aid]['y'] -
+                            data_animal_id_groups[aid]['y'].shift(periods=1)),
+                           (data_animal_id_groups[aid]['x'] -
+                            data_animal_id_groups[aid]['x'].shift(periods=1))))
+        except TypeError:
+            data_animal_id_groups[aid]['direction'] = 0
 
     # Compute 'distance' for 'animal_id' groups-
     for aid in data_animal_id_groups.keys():
-        p1 = data_animal_id_groups[aid].loc[:, ['x', 'y']]
-        p2 = data_animal_id_groups[aid].loc[:, ['x', 'y']].shift(periods=1)
-        p2.iloc[0, :] = [0.0, 0.0]
+        try:
+            p1 = data_animal_id_groups[aid].loc[:, ['x', 'y']]
+            p2 = data_animal_id_groups[aid].loc[:, ['x', 'y']].shift(periods=1)
+            p2.iloc[0, :] = [0.0, 0.0]
 
-        data_animal_id_groups[aid]['distance'] = ((p1 -
-                                                   p2)**2).sum(axis=1)**0.5
+            data_animal_id_groups[aid]['distance'] = ((p1 -
+                                                       p2)**2).sum(axis=1)**0.5
+
+        except TypeError:
+            data_animal_id_groups[aid]['distance'] = 0
 
     # Reset first entry for each 'animal_id' to zero-
     for aid in data_animal_id_groups.keys():
@@ -163,9 +171,9 @@ def compute_average_speed(data_animal_id_groups, fps):
     :return: dictionary, including measure for 'average_speed'
     """
     for aid in data_animal_id_groups.keys():
-        data_animal_id_groups[aid]['average_speed'] = data_animal_id_groups[aid] \
-                                                          ['distance'].rolling(window=fps, win_type=None).sum() / fps
-
+        data_animal_id_groups[aid]['average_speed'] = data_animal_id_groups[
+            aid]['distance'].rolling(min_periods=1, window=fps,
+                                     center=True).mean().fillna(0)
     return data_animal_id_groups
 
 
@@ -179,37 +187,20 @@ def compute_average_acceleration(data_animal_id_groups, fps):
     :return: dictionary, including measure for 'average_acceleration'
     """
     for aid in data_animal_id_groups.keys():
-        a = data_animal_id_groups[aid]['average_speed']
-        b = data_animal_id_groups[aid]['average_speed'].shift(periods=1)
 
-        data_animal_id_groups[aid]['average_acceleration'] = (a - b) / fps
+        # rename into shortcut
+        speed = data_animal_id_groups[aid]['average_speed']
+        #b = data_animal_id_groups[aid]['average_speed'].shift(periods=1)
+        try:
+            data_animal_id_groups[aid]['average_acceleration'] = speed.rolling(
+                min_periods=1, window=fps,
+                center=True).apply(lambda x: x[1] - x[0], raw=True).fillna(0)
+        except:
+            data_animal_id_groups[aid]['average_acceleration'] = 0
+
     return data_animal_id_groups
 
 
-def compute_absolute_features(data_animal_id_groups,
-                              fps=10,
-                              stop_threshold=0.5):
-    """
-    Calculate absolute features for the input data animal group.
-    Combined usage of the functions on dictionary 'compute_average_speed(data,fps)', 'compute_average_acceleration(data,fps)' and
-    'computing_stops(data, threshold)'.
-    :param data_animal_id_groups: dictionary with 'animal_id' as keys.
-    :param fps: integer to specify frames per second.
-    :param stop_threshold: integer to specify threshold, at which we consider a "stop".
-    :return: dictionary with additional variables 'avg_speed_data', 'avg_acceleration_data' and 'stop data'.
-    """
-    direction_distance_data = compute_distance_and_direction(
-        data_animal_id_groups)
-
-    avg_speed_data = compute_average_speed(direction_distance_data, fps)
-
-    avg_acceleration_data = compute_average_acceleration(avg_speed_data, fps)
-
-    stop_data = computing_stops(avg_acceleration_data, stop_threshold)
-
-    return stop_data
-
-
 def extract_features(data, fps=10, stop_threshold=0.5):
     """
     Calculate and return all absolute features for input animal group.
@@ -258,7 +249,48 @@ def computing_stops(data_animal_id_groups, threshold_speed):
     '''
 
 
-def medoid_computation(data, only_centroid=False):
+def group_movement(feats):
+    """
+    Returns aggregated movement data, such as distance, mean speed, mean acceleration and mean distance to centroid for the entire group at each time capture.
+
+    :param feats: pd DataFrame with animal-specific data - if no features contained, they will be extracted.
+    :return: pd DataFrame with group-specific values for each time-capture
+
+    """
+
+    # Handling no features in input
+    if 'distance' not in feats.columns or 'average_speed' not in feats.columns or 'average_acceleration' not in feats.columns:
+        warnings.warn(
+            'Recalculating features, since distance, speed or acceleration not found in input.'
+        )
+        feats = extract_features(feats)
+
+    # Handling no centroid in input
+    if 'distance_to_centroid' not in feats.columns:
+        warnings.warn(
+            'Recalculating centroid-distances, since not found in input dataset!'
+        )
+        feats = centroid_medoid_computation(feats)
+    # Group by time, return new dataframe
+    data_dist = feats.groupby('time')
+
+    group = pd.DataFrame({
+        "total_dist":
+        data_dist.sum()['distance'],
+        "mean_speed":
+        data_dist.mean()['average_speed'],
+        "mean_acceleration":
+        data_dist.mean()['average_acceleration'],
+        "mean_distance_centroid":
+        data_dist.mean()['distance_to_centroid']
+    })
+
+    return group
+
+
+def centroid_medoid_computation(data,
+                                only_centroid=False,
+                                object_output=False):
     """
     Calculates the data point (animal_id) closest to center/centroid/medoid for a time step
     Uses group by on 'time' attribute
@@ -298,6 +330,11 @@ def medoid_computation(data, only_centroid=False):
         data_groups_time[aid] = data_groups_time[aid].assign(x_centroid=x_mean)
         data_groups_time[aid] = data_groups_time[aid].assign(y_centroid=y_mean)
 
+        # add additional variable for point object
+        if object_output == True:
+            data_groups_time[aid] = data_groups_time[aid].assign(
+                centroid=functions.ST_MakePoint(x_mean, y_mean))
+
         if only_centroid == False:
             # Squared distance of each 'x' coordinate to 'centroid'-
             x_temp = (data_groups_time[aid].loc[:, 'x'] - x_mean)**2
